@@ -129,6 +129,10 @@ TEST_P(WithAndWithoutComputeShaderTest, TopLeftOrigin) {
 	EffectChainTester tester(data, 3, 2, FORMAT_GRAYSCALE, COLORSPACE_sRGB, GAMMA_LINEAR);
 	tester.get_chain()->set_output_origin(OUTPUT_ORIGIN_TOP_LEFT);
 	if (GetParam() == "compute") {
+		if (!movit_compute_shaders_supported) {
+			fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+			return;
+		}
 		tester.get_chain()->add_effect(new IdentityComputeEffect());
 	}
 	tester.run(out_data, GL_RED, COLORSPACE_sRGB, GAMMA_LINEAR);
@@ -791,7 +795,6 @@ public:
 
 private:
 	const MipmapRequirements mipmap_requirements;
-	EffectChain *chain;
 	float offset[2] { 0.0f, 0.0f };
 };
 
@@ -1295,6 +1298,10 @@ TEST_P(WithAndWithoutComputeShaderTest, NoBounceWithOneToOneSampling) {
 	RewritingEffect<OneToOneEffect> *effect2 = new RewritingEffect<OneToOneEffect>();
 
 	if (GetParam() == "compute") {
+		if (!movit_compute_shaders_supported) {
+			fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+			return;
+		}
 		tester.get_chain()->add_effect(new IdentityComputeEffect());
 	} else {
 		tester.get_chain()->add_effect(new NonVirtualResizeEffect(size, size));
@@ -1549,6 +1556,10 @@ TEST_P(WithAndWithoutComputeShaderTest, SquareRoot10bitIntermediateAccuracy) {
 	EffectChainTester tester(data, size, 1, FORMAT_GRAYSCALE, COLORSPACE_sRGB, GAMMA_LINEAR, GL_RGBA32F);
 	tester.get_chain()->set_intermediate_format(GL_RGB10_A2, SQUARE_ROOT_FRAMEBUFFER_TRANSFORMATION);
 	if (GetParam() == "compute") {
+		if (!movit_compute_shaders_supported) {
+			fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+			return;
+		}
 		tester.get_chain()->add_effect(new IdentityComputeEffect());
 	} else {
 		tester.get_chain()->add_effect(new IdentityEffect());
@@ -1718,6 +1729,10 @@ TEST(ComputeShaderTest, ComputeThenOneToOne) {
 	};
 	float out_data[8];
 	EffectChainTester tester(data, 4, 2, FORMAT_GRAYSCALE, COLORSPACE_sRGB, GAMMA_LINEAR);
+	if (!movit_compute_shaders_supported) {
+		fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+		return;
+	}
 	tester.get_chain()->add_effect(new MirrorComputeEffect());
 	tester.get_chain()->add_effect(new OneToOneEffect());
 	tester.run(out_data, GL_RED, COLORSPACE_sRGB, GAMMA_LINEAR);
@@ -1761,6 +1776,10 @@ TEST(ComputeShaderTest, ResizingComputeThenOneToOne) {
 	};
 	float out_data[2];
 	EffectChainTester tester(nullptr, 2, 1);
+	if (!movit_compute_shaders_supported) {
+		fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+		return;
+	}
 	tester.add_input(data, FORMAT_GRAYSCALE, COLORSPACE_sRGB, GAMMA_LINEAR, 4, 2);
 
 	RewritingEffect<Downscale2xComputeEffect> *downscale_effect = new RewritingEffect<Downscale2xComputeEffect>();
@@ -1798,6 +1817,10 @@ TEST(ComputeShaderTest, NoTwoComputeInSamePhase) {
 	format.gamma_curve = GAMMA_LINEAR;
 
 	EffectChainTester tester(nullptr, 2, 1);
+	if (!movit_compute_shaders_supported) {
+		fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+		return;
+	}
 
 	FlatInput *input1 = new FlatInput(format, FORMAT_GRAYSCALE, GL_FLOAT, 4, 2);
 	input1->set_pixel_data(data);
@@ -1831,6 +1854,10 @@ TEST(ComputeShaderTest, NoTwoComputeInSamePhaseIndirect) {
 	format.gamma_curve = GAMMA_LINEAR;
 
 	EffectChainTester tester(nullptr, 2, 1);
+	if (!movit_compute_shaders_supported) {
+		fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+		return;
+	}
 
 	FlatInput *input1 = new FlatInput(format, FORMAT_GRAYSCALE, GL_FLOAT, 4, 2);
 	input1->set_pixel_data(data);
@@ -1867,6 +1894,10 @@ TEST(ComputeShaderTest, BounceTextureFromTwoComputeShaders) {
 	format.gamma_curve = GAMMA_LINEAR;
 
 	EffectChainTester tester(nullptr, 2, 1);
+	if (!movit_compute_shaders_supported) {
+		fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+		return;
+	}
 
 	FlatInput *input1 = new FlatInput(format, FORMAT_GRAYSCALE, GL_FLOAT, 4, 2);
 	input1->set_pixel_data(data);
@@ -1906,6 +1937,10 @@ TEST(ComputeShaderTest, StrongOneToOneButStillNotChained) {
 	format.gamma_curve = GAMMA_LINEAR;
 
 	EffectChainTester tester(nullptr, 4, 2);
+	if (!movit_compute_shaders_supported) {
+		fprintf(stderr, "Skipping test; no support for compile shaders.\n");
+		return;
+	}
 
 	FlatInput *input1 = new FlatInput(format, FORMAT_GRAYSCALE, GL_FLOAT, 4, 2);
 	input1->set_pixel_data(data);
@@ -1925,4 +1960,42 @@ TEST(ComputeShaderTest, StrongOneToOneButStillNotChained) {
 	expect_equal(data, out_data, 4, 2);
 }
 
+TEST(EffectChainTest, BounceResetsMipmapNeeds) {
+	float data[] = {
+		0.0f, 0.25f,
+		0.75f, 1.0f,
+	};
+	float out_data[1];
+
+	ImageFormat format;
+	format.color_space = COLORSPACE_sRGB;
+	format.gamma_curve = GAMMA_LINEAR;
+
+	NonMipmapCapableInput *input = new NonMipmapCapableInput(format, FORMAT_GRAYSCALE, GL_FLOAT, 2, 2);
+	input->set_pixel_data(data);
+
+	RewritingEffect<IdentityEffect> *identity = new RewritingEffect<IdentityEffect>();
+
+	RewritingEffect<ResizeEffect> *downscale = new RewritingEffect<ResizeEffect>();  // Needs mipmaps.
+	ASSERT_TRUE(downscale->effect->set_int("width", 1));
+	ASSERT_TRUE(downscale->effect->set_int("height", 1));
+
+	EffectChainTester tester(nullptr, 1, 1);
+	tester.get_chain()->add_input(input);
+	tester.get_chain()->add_effect(identity);
+	tester.get_chain()->add_effect(downscale);
+	tester.run(out_data, GL_RED, COLORSPACE_sRGB, GAMMA_LINEAR);
+
+	Node *input_node = identity->replaced_node->incoming_links[0];
+
+	// The ResizeEffect needs mipmaps. Normally, that would mean that it should
+	// propagate this tatus down through the IdentityEffect. However, since we
+	// bounce (due to the resize), the dependency breaks there, and we don't
+	// need to bounce again between the input and the IdentityEffect.
+	EXPECT_EQ(input_node->containing_phase,
+	          identity->replaced_node->containing_phase);
+	EXPECT_NE(identity->replaced_node->containing_phase,
+	          downscale->replaced_node->containing_phase);
+}
+
 }  // namespace movit
