@@ -17,7 +17,9 @@ enum MovitDebugLevel {
 // succeeded.
 //
 // The first parameter gives which directory to read .frag files from.
-// This is a temporary hack until we add something more solid.
+// If you use "", or the file isn't found, a compiled-in version will
+// be used instead (if it exists). Most users should probably use ""
+// unless you need backwards compatibility with Movit 1.6.3 or older.
 //
 // The second parameter specifies whether debugging is on or off.
 // If it is on, Movit will write intermediate graphs and the final
@@ -50,32 +52,34 @@ extern float movit_texel_subpixel_precision;
 
 // Some GPUs use very inaccurate fixed-function circuits for rounding
 // floating-point values to 8-bit outputs, leading to absurdities like
-// the roundoff point between 128 and 129 being 128.62 instead of 128.6.
+// the roundoff point between 128 and 129 being 128.62 instead of 128.5.
 // We test, for every integer, x+0.48 and x+0.52 and check that they
 // round the right way (giving some leeway, but not a lot); the number
 // of errors are stored here.
 //
-// If this value is above 0, the extension GL_EXT_gpu_shader4 is available
-// (giving round()) and you have enabled dithering, we will round off
-// explicitly at the very end of the shader.
+// If this value is above 0, we will round off explicitly at the very end
+// of the shader. Note the following limitations:
 //
-// Note: I don't know of any cards that round off wrong (well, outside
-// our tolerance) and do not have this extension.
+//   - The measurement is done on linear 8-bit, not any sRGB format,
+//     10-bit output, or the likes.
+//   - This only covers the final pass; intermediates are not covered
+//     (only relevant if you use e.g. GL_SRGB8 intermediates).
 extern int movit_num_wrongly_rounded;
-extern bool movit_shader_rounding_supported;
-
-// Whether the GPU in use supports GL_EXT_texture_sRGB.
-extern bool movit_srgb_textures_supported;
 
 // Whether the OpenGL driver (or GPU) in use supports GL_ARB_timer_query.
 extern bool movit_timer_queries_supported;
 
+// Whether the OpenGL driver (or GPU) in use supports compute shaders.
+// Note that certain OpenGL implementations might only allow this in core mode.
+extern bool movit_compute_shaders_supported;
+
 // What shader model we are compiling for. This only affects the choice
 // of a few files (like header.frag); most of the shaders are the same.
 enum MovitShaderModel {
-	MOVIT_GLSL_110,
+	MOVIT_GLSL_110,  // No longer in use, but kept until next ABI break in order not to change the enums.
 	MOVIT_GLSL_130,
-	MOVIT_ESSL_300
+	MOVIT_ESSL_300,
+	MOVIT_GLSL_150,
 };
 extern MovitShaderModel movit_shader_model;
 
