@@ -16,10 +16,10 @@ namespace movit {
 class SaturationEffect : public Effect {
 public:
 	SaturationEffect();
-	virtual std::string effect_type_id() const { return "SaturationEffect"; }
-	virtual AlphaHandling alpha_handling() const { return DONT_CARE_ALPHA_TYPE; }
-	virtual bool one_to_one_sampling() const { return true; }
-	std::string output_fragment_shader();
+	std::string effect_type_id() const override { return "SaturationEffect"; }
+	AlphaHandling alpha_handling() const override { return DONT_CARE_ALPHA_TYPE; }
+	bool strong_one_to_one_sampling() const override { return true; }
+	std::string output_fragment_shader() override;
 
 private:
 	float saturation;
