@@ -41,7 +41,7 @@ TEST(YCbCrInputTest, Simple444) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -92,7 +92,7 @@ TEST(YCbCrInputTest, Interleaved444) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -145,7 +145,7 @@ TEST(YCbCrInputTest, FullRangeRec601) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -199,7 +199,7 @@ TEST(YCbCrInputTest, Rec709) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -255,7 +255,7 @@ TEST(YCbCrInputTest, Rec2020) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -310,7 +310,7 @@ TEST(YCbCrInputTest, ChangeFormat) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -391,7 +391,7 @@ TEST(YCbCrInputTest, Subsampling420) {
 	};
 	float out_data[width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -453,7 +453,7 @@ TEST(YCbCrInputTest, Subsampling420WithNonCenteredSamples) {
 	};
 	float out_data[width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -523,7 +523,7 @@ TEST(YCbCrInputTest, DifferentCbAndCrPositioning) {
 	};
 	float out_data[width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -581,7 +581,7 @@ TEST(YCbCrInputTest, PBO) {
 	glBufferData(GL_PIXEL_UNPACK_BUFFER_ARB, width * height * 3, data, GL_STREAM_DRAW);
 	glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, 0);
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -638,7 +638,7 @@ TEST(YCbCrInputTest, CombinedCbAndCr) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -691,7 +691,7 @@ TEST(YCbCrInputTest, ExternalTexture) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -849,7 +849,7 @@ TEST(YCbCrInputTest, NoData) {
 
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -906,7 +906,7 @@ TEST(YCbCrInputTest, TenBitInterleaved) {
 			(expanded_data[i * 3 + 2] << 20);
 	}
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -969,7 +969,7 @@ TEST(YCbCrInputTest, TenBitPlanar) {
 	};
 	float out_data[4 * width * height];
 
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 
 	ImageFormat format;
 	format.color_space = COLORSPACE_sRGB;
@@ -1004,16 +1004,16 @@ TEST(YCbCrInputTest, TenBitPlanar) {
 class MipmapNeedingEffect : public Effect {
 public:
 	MipmapNeedingEffect() {}
-	virtual bool needs_mipmaps() const { return true; }
+	MipmapRequirements needs_mipmaps() const override { return NEEDS_MIPMAPS; }
 
 	// To be allowed to mess with the sampler state.
-	virtual bool needs_texture_bounce() const { return true; }
+	bool needs_texture_bounce() const override { return true; }
 
-	virtual string effect_type_id() const { return "MipmapNeedingEffect"; }
-	string output_fragment_shader() { return read_file("mipmap_needing_effect.frag"); }
-	virtual void inform_added(EffectChain *chain) { this->chain = chain; }
+	string effect_type_id() const override { return "MipmapNeedingEffect"; }
+	string output_fragment_shader() override { return read_file("mipmap_needing_effect.frag"); }
+	void inform_added(EffectChain *chain) override { this->chain = chain; }
 
-	void set_gl_state(GLuint glsl_program_num, const string& prefix, unsigned *sampler_num)
+	void set_gl_state(GLuint glsl_program_num, const string& prefix, unsigned *sampler_num) override
 	{
 		Node *self = chain->find_node_for_effect(this);
 		glActiveTexture(chain->get_input_sampler(self, 0));
@@ -1031,8 +1031,8 @@ private:
 // Basically the same test as EffectChainTest_MipmapGenerationWorks,
 // just with the data converted to Y'CbCr (as red only).
 TEST(EffectChainTest, MipmapGenerationWorks) {
-	unsigned width = 4;
-	unsigned height = 16;
+	const unsigned width = 4;
+	const unsigned height = 16;
 	float red_data[width * height] = {  // In 4x4 blocks.
 		1.0f, 0.0f, 0.0f, 0.0f,
 		0.0f, 0.0f, 0.0f, 0.0f,
@@ -1119,7 +1119,7 @@ TEST(EffectChainTest, MipmapGenerationWorks) {
 	format.gamma_curve = GAMMA_sRGB;
 
 	float out_data[width * height * 4];
-	EffectChainTester tester(NULL, width, height);
+	EffectChainTester tester(nullptr, width, height);
 	YCbCrInput *input = new YCbCrInput(format, ycbcr_format, width, height, YCBCR_INPUT_INTERLEAVED);
 	input->set_pixel_data(0, ycbcr_data);
 	tester.get_chain()->add_input(input);
